/*
** Filename: AOAtoFT232.c
**
** Automatically created by Application Wizard 2.0.0
** 
** Part of solution AOAtoFT232 in project AOAtoFT232
**
** Comments: 
** THIS SOFTWARE IS PROVIDED BY FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED
** ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
** TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL FUTURE TECHNOLOGY DEVICES
** INTERNATIONAL LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
** EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
** OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
** STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGE.
*/

#include "AOAtoFT232.h"

vos_tcb_t *tcbATOF;
vos_tcb_t *tcbFTOA;
vos_tcb_t *tcbCONNECT;

void atof();							// Thread to copy data from Android device to FT232
void ftoa();							// Thread to copy data from FT232 do Android device
void connect();							// Thread to establish and tear-down USB connections

VOS_HANDLE hUSBHOST_1; 					// USB Host Port 1 - connect to Android device
VOS_HANDLE hUSBHOST_2; 					// USB Host Port 2 - connect to FT232
VOS_HANDLE hGPIO_PORT_A; 				// GPIO Port A Driver - Used for LEDs
VOS_HANDLE hANDROID_ACCESSORY; 			// Android Open Accessory Class Driver
VOS_HANDLE hUSBHOST_FT232; 				// Connects to FT232 devices using the USB Host Interface

vos_mutex_t mInitA;						// Mutex to halt AOA activity until setup is complete
vos_mutex_t mInitF;						// Mutex to halt FT232 activity until setup is complete

unsigned char r232wAOA[64];
unsigned char rAOAw232[64];
unsigned int  ft232Baud = USBHOSTFT232_BAUD_9600;		// Set up default FT232 UART parameters
unsigned char ft232Data = USBHOSTFT232_DATA_BITS_8;
unsigned char ft232Stop = USBHOSTFT232_STOP_BITS_1;
unsigned char ft232Parity = USBHOSTFT232_PARITY_NONE;
unsigned char ft232Flow = USBHOSTFT232_FLOW_NONE;

/* Declaration for IOMUx setup function */
void iomux_setup(void);

/* Main code - entry point to firmware */
void main(void)
{
	/* FTDI:SDD Driver Declarations */
	// GPIO Port A configuration context
	gpio_context_t gpioContextA;
	// USB Host configuration context
	usbhost_context_t usbhostContext;
	/* FTDI:EDD */

	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(512);
	/* FTDI:EKI */

	iomux_setup();						// Configure the IO pins - just three for LEDs and the debug interface

	/* FTDI:SDI Driver Initialisation */
	// Initialise GPIO A
	gpioContextA.port_identifier = GPIO_PORT_A;
	gpio_init(VOS_DEV_GPIO_PORT_A,&gpioContextA);
	
	// Initialise Android Accessory Device Driver
	usbHostAndroidAccessory_init(VOS_DEV_ANDROID_ACCESSORY);
	
	// Initialise USB Host FT232 Driver
	usbHostFt232_init(VOS_DEV_USBHOST_FT232);
	
	// Initialise USB Host
	usbhostContext.if_count = 8;
	usbhostContext.ep_count = 16;
	usbhostContext.xfer_count = 2;
	usbhostContext.iso_xfer_count = 2;
	usbhost_init(VOS_DEV_USBHOST_1, VOS_DEV_USBHOST_2, &usbhostContext);

	// Create threads
	tcbATOF = vos_create_thread_ex(24, 1024, atof, "atof", 0);
	tcbFTOA = vos_create_thread_ex(24, 1024, ftoa, "ftoa", 0);
	tcbCONNECT = vos_create_thread_ex(24, 2048, connect, "connect", 0);
	
	// Initialize Mutexes as locked
	vos_init_mutex(&mInitA, VOS_MUTEX_LOCKED);
	vos_init_mutex(&mInitF, VOS_MUTEX_LOCKED);
	
	vos_start_scheduler();

main_loop:
	goto main_loop;
}


// The following functions are genereated by the new project wizard
/* FTDI:SSP Support Functions */

unsigned char usbhost_connect_state(VOS_HANDLE hUSB)
{
	unsigned char connectstate = PORT_STATE_DISCONNECTED;
	usbhost_ioctl_cb_t hc_iocb;

	if (hUSB)
	{
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_GET_CONNECT_STATE;
		hc_iocb.get        = &connectstate;
		vos_dev_ioctl(hUSB, &hc_iocb);

    // repeat if connected to see if we move to enumerated
		if (connectstate == PORT_STATE_CONNECTED)
		{
			vos_dev_ioctl(hUSB, &hc_iocb);
		}
	}
	return connectstate;
}


VOS_HANDLE android_attach(VOS_HANDLE hUSB, unsigned char devANDACC,
	char *manufacturer, char *model, char *description,
	char *version, char *uri, char *serial)
{
	common_ioctl_cb_t androidAccessory_cb;
	usbHostAndroidAccessory_ioctl_cb_attach_t atInfo;
	VOS_HANDLE hANDACC;
	unsigned char status = NULL;
	unsigned short protocolRevision = NULL;

	hANDACC = vos_dev_open(devANDACC);

	atInfo.hc_handle = hUSB;
	atInfo.manufacturer = manufacturer;
	atInfo.model = model;
	atInfo.description = description;
	atInfo.version = version;
	atInfo.uri = uri;
	atInfo.serial = serial;

	androidAccessory_cb.ioctl_code = VOS_IOCTL_USBHOSTANDROIDACCESSORY_ATTACH;
	androidAccessory_cb.set.data = &atInfo;
	if (vos_dev_ioctl(hANDACC, &androidAccessory_cb) != USBHOSTANDROIDACCESSORY_OK)
	{
		vos_dev_close(hANDACC);
		hANDACC = NULL;
	}

	return hANDACC;
}

void android_detach(VOS_HANDLE hANDACC)
{
	common_ioctl_cb_t androidAccessory_cb;

	if (hANDACC)
	{
		androidAccessory_cb.ioctl_code = VOS_IOCTL_USBHOSTANDROIDACCESSORY_DETACH;
		vos_dev_ioctl(hANDACC, &androidAccessory_cb);
		vos_dev_close(hANDACC);
	}
}


VOS_HANDLE ft232_host_attach(VOS_HANDLE hUSB, unsigned char devHostFT232, unsigned char ftport)
{
	usbhost_device_handle_ex ifFT232 = 0;
	usbhost_ioctl_cb_t hc_iocb;
	usbhost_ioctl_cb_vid_pid_t hc_iocb_vendor;
	usbhostft232_ioctl_cb_attach_t ft232_att;
	common_ioctl_cb_t ft232_iocb;
	VOS_HANDLE hHostFT232;
	unsigned char status;

	// find FT232 class device
	hc_iocb_vendor.vid = USB_VID_FTDI;
	hc_iocb_vendor.pid = USB_PID_FTDI_FT232;

	// user ioctl to find first FT232 device
	hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_FIND_HANDLE_BY_VID_PID;
	hc_iocb.handle.dif = NULL;
	hc_iocb.set = &hc_iocb_vendor;
	hc_iocb.get = &ifFT232;

	if (vos_dev_ioctl(hUSB, &hc_iocb) != USBHOST_OK)
	{
		return NULL;
	}

	// now we have a device, intialise the FT232 driver for it
	hHostFT232 = vos_dev_open(devHostFT232);

	// perform attach
	ft232_att.hc_handle = hUSB;
	ft232_att.ifDev = ifFT232;
	ft232_att.ftPort = ftport;

	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_ATTACH;
	ft232_iocb.set = &ft232_att;
	ft232_iocb.get = NULL;

	if (vos_dev_ioctl(hHostFT232, &ft232_iocb) != USBHOSTFT232_OK)
	{
		vos_dev_close(hHostFT232);
		hHostFT232 = NULL;
	}

	// Configure FT232
	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_BAUD_RATE;
	ft232_iocb.set.uart_baud_rate = ft232Baud;
	status = vos_dev_ioctl(hHostFT232, &ft232_iocb);

	// FT232 set flow control
	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_FLOW_CONTROL;
	ft232_iocb.set.param = ft232Flow;
	status = vos_dev_ioctl(hHostFT232, &ft232_iocb);

	// FT232 set data bits
	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_DATA_BITS;
	ft232_iocb.set.param = ft232Data;
	status = vos_dev_ioctl(hHostFT232, &ft232_iocb);

	// FT232 set stop bits
	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_STOP_BITS;
	ft232_iocb.set.param = ft232Stop;
	status = vos_dev_ioctl(hHostFT232, &ft232_iocb);

	// FT232 set parity
	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_PARITY;
	ft232_iocb.set.param = ft232Parity;
	status = vos_dev_ioctl(hHostFT232, &ft232_iocb);

	// start polling to ensure we receive data in the RX Queue
	ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_START_POLL;
	status = vos_dev_ioctl(hHostFT232, &ft232_iocb);
	
	return hHostFT232;
}

void ft232_host_detach(VOS_HANDLE hHostFT232)
{
	common_ioctl_cb_t ft232_iocb;

	if (hHostFT232)
	{
		ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_DETACH;

		vos_dev_ioctl(hHostFT232, &ft232_iocb);
		vos_dev_close(hHostFT232);
	}
}

/* Application Threads */

/*
	Thread Name:  connect()
	Comments:
	This thread establishes the following USB connections:
		USB1 = Connection to an Android Device with Android Open Accessory support
			   The "FTDI AOA HyperTerm" must be loaded from the Google Play Store
		USB2 = Connection to a FT232B- or FT232R-based RS232 cable (FT23xX support 
			   is not present but can be added with the appropriate USB Product ID
			   recognition
	Upon connection of both devices, data is copied in both directions by the "atof()"
	and "ftoa()" threads.  Upon detection of a disconnect from either USB port, 
	all USB connections are terminated and the VNC2 halted.  
*/
void connect()
{
	common_ioctl_cb_t aoa_iocb;
	common_ioctl_cb_t ft232_iocb;
		
	// Android Accessory Driver attach must specify various strings
	unsigned char status;
	unsigned char blink = 0;
	
	unsigned short protocolRevision = 0;
	unsigned short actual, actualw, dataAvail = 0;
	
	char *manufacturer 	= "FTDI\0";						// Android Open Accessory ID strings
	char *model 		= "FTDIUARTDemo\0";				// These need to match the strings
	char *description 	= "Vinculum Accessory Test\0";	//  expected by the FTDI AOA HyperTerm
	char *version 		= "1.0\0";						//  application on the Android device
	char *uri 			= "http://www.ftdichip.com\0";
	char *serial 		= "VinculumAccessory1\0";
	
	hGPIO_PORT_A = vos_dev_open(VOS_DEV_GPIO_PORT_A);	// Open up the GPIO port - used for LEDs
	vos_gpio_write_port(GPIO_PORT_A, 0xFF);				// Pre-set PORT A all high - prevents glitch
	vos_gpio_set_port_mode(GPIO_PORT_A, 0xFF); 			// Port A all output
	vos_gpio_write_pin(GPIO_A_3, 0);					// Turn on LED1 on V2EVAL
	
	// Open handles
	hUSBHOST_1 = vos_dev_open(VOS_DEV_USBHOST_1);		// Open USB Host Port 1
	hUSBHOST_2 = vos_dev_open(VOS_DEV_USBHOST_2);		// Open USB Host Port 2
	// Wait for both to connect
	vos_delay_msecs(100);
	while(!usbhost_connect_state(hUSBHOST_1) || !usbhost_connect_state(hUSBHOST_2))	// Wait for both devices to connect
	{
		vos_gpio_write_pin(GPIO_A_3, blink);			// While waiting, blink LED1 at 10Hz
		blink = !blink;
		vos_delay_msecs(100);
	}
		
	// Attach layered drivers
	hANDROID_ACCESSORY = android_attach(hUSBHOST_1, VOS_DEV_ANDROID_ACCESSORY, manufacturer, model, description, version, uri, serial);
	if(hANDROID_ACCESSORY)
		vos_gpio_write_pin(GPIO_A_1, 0);				// Android device connected and AOA mode established - Turn on LED at USB1
		
	hUSBHOST_FT232 = ft232_host_attach(hUSBHOST_2, VOS_DEV_USBHOST_FT232, USBHOSTFT232_PORTA);
	if(hUSBHOST_FT232)
		vos_gpio_write_pin(GPIO_A_2, 0);				// FT232 connected - Turn on LED at USB2
		
	vos_gpio_write_pin(GPIO_A_3, 1);					// Turn off LED1 while everything is connected

	vos_delay_msecs(500);								// Give a little time for the USB devices to get initialized
		
	status = vos_dev_write(hANDROID_ACCESSORY, "\n\rSend Config through Settings button first\r\n", 45, &actual);	// send initial message 
	while(status)										// The default setting is 9600,n,8,1,no handshake
	{
		vos_gpio_write_pin(GPIO_A_3, blink);			// Something's wrong with the AOA connection.  
		blink = !blink;
		vos_delay_msecs(50);							// Blink LED1 at 20Hz forever since "status" will not change at this point
	}
		
	status = vos_dev_read(hANDROID_ACCESSORY, rAOAw232, sizeof(rAOAw232), &actual); // Read in the configuration string 
														// This call will block until some data is received from the Android device

	ft232Baud = (rAOAw232[3]<<24 | rAOAw232[2]<<16 | rAOAw232[1]<<8 | rAOAw232[0]);	// Parse the baud rate from the configuration string
	
	if (rAOAw232[4] == 7)								// Parse the number of data bits
		ft232Data = 0;									// 		7 data bits
	else
		ft232Data = 1;									//		8 data bits
	
	if (rAOAw232[5] == 1)								// Parse the number of stop bits
		ft232Stop = 0;									// 		1 stop bit
	else
		ft232Stop = 1;									//		2 stop bits
	
	ft232Parity = rAOAw232[6];							// Parse the parity setting
	
	ft232Flow = rAOAw232[7];							// Parse the flow control setting
														// NOTE: Be sure to set whatever is connected to the FT232 to these NEW settings
	
	ft232_host_detach(hUSBHOST_FT232);					// detach the FT232...
														// ... and re-attach with the new values parsed above
	hUSBHOST_FT232 = ft232_host_attach(hUSBHOST_2, VOS_DEV_USBHOST_FT232, USBHOSTFT232_PORTA);
	
	if (ft232Flow == USBHOSTFT232_FLOW_RTS_CTS)
	{
		ft232_iocb.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_RTS;	// If flow is set to RTS/CTS, then set RTS on the FT232
		status = vos_dev_ioctl(hUSBHOST_FT232, &ft232_iocb);	// to allow connected device to send data
	}

	vos_unlock_mutex(&mInitA);							// Allow the "atof()" thread to start
	vos_unlock_mutex(&mInitF);							// Allow the "ftoa()" thread to start
		
	while(usbhost_connect_state(hUSBHOST_1) && usbhost_connect_state(hUSBHOST_2)) // spin in this loop as long as both ports are connected
	{
		vos_delay_msecs(500);							// Blink LED1 at 2Hz as a "heartbeat"
		blink = !blink;
		vos_gpio_write_pin(GPIO_A_3, blink);
	}

	// If we drop out of the while() loop, one of the USB ports was unplugged
	
	if (usbhost_connect_state(hUSBHOST_1))				// If the Android device is still connected... 
		vos_dev_write(hANDROID_ACCESSORY, "\n\rFT232 is UNPLUGGED - Halting VNC2!\n\r", 38, &actual);//... send a note indicating the FT232 is gone
	
	android_detach(hANDROID_ACCESSORY);					// Break down the Android device connection
	ft232_host_detach(hUSBHOST_FT232);					// Break down the FT232 connection
	vos_gpio_write_pin(GPIO_A_1, 1);					// Turn off LED at USB1
	vos_gpio_write_pin(GPIO_A_2, 1);					// Turn off LED at USB2
	vos_dev_close(hUSBHOST_1);							// Close the USB handle to the Android device
	vos_dev_close(hUSBHOST_2);							// Close the USB handle to the FT232
	hUSBHOST_1 = NULL;									// Housekeeping - set handles to zero
	hANDROID_ACCESSORY = NULL;
	hUSBHOST_2 = NULL;
	hUSBHOST_FT232 = NULL;
	vos_gpio_write_pin(GPIO_A_3, 0);					// Turn on LED1
	vos_halt_cpu();										// Stop here - a VNC2 reset is required to start again
}

/*
	Thread Name:  atof()
	Comments:
	This thread reads data from the Android device on USB1 and copies it to the FT232 on USB2
*/	
void atof()
{
	common_ioctl_cb_t aoa_iocb;
	unsigned char status, i;
	unsigned short dataAvail = 0, actual = 0, actualw = 0;

	while(1)
	{
		vos_lock_mutex(&mInitA);						// hold here until unlocked - lock then proceed

		// VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS is not a valid ioctl for Android Open Accessory mode
		// Just let vos_dev_read block until some data comes in.
		
		status = vos_dev_read(hANDROID_ACCESSORY, rAOAw232, sizeof(rAOAw232), &actual);	// read from Android device 
		status = vos_dev_write(hUSBHOST_FT232, rAOAw232, actual, &actualw);				// then write to the FT232
		vos_unlock_mutex(&mInitA);						// unlock the AOA mutex
	}
}

/*
	Thread Name:  ftoa()
	Comments:
	This thread reads data from the FT232 on USB2 and copies it to the Android device on USB1
*/		
void ftoa()
{
	common_ioctl_cb_t ft232_iocb;
	unsigned char status, i;
	unsigned short dataAvail = 0, actual = 0, actualw = 0;
	
	while(1)
	{
		vos_lock_mutex(&mInitF);						// hold here until unlocked - lock then proceed

		ft232_iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;					// check whether any data is 
		status = vos_dev_ioctl(hUSBHOST_FT232, &ft232_iocb);							// available at from the FT232
		dataAvail = ft232_iocb.get.queue_stat;

		if (dataAvail > sizeof(r232wAOA))				// If there is more data than the buffer size.
		{
			dataAvail = sizeof(r232wAOA); 				// ... then limit the number of bytes to the buffer size
		}
		if (dataAvail > 0)								// If there is any data available
		{
			status = vos_dev_read(hUSBHOST_FT232, r232wAOA, dataAvail, &actual); 		// read from FT232 device
			status = vos_dev_write(hANDROID_ACCESSORY, r232wAOA, actual, &actualw);		// then write to Android device
		}
		vos_unlock_mutex(&mInitF);						// unlock the FT232 mutex
	}
}
// EOF
